#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>

#include "serial.h"
#include "SPI.h"
#include "enc28j60.h"
#include "SX1276.h"

#define PACKET_FIN	0x80
#define MAX_FRAGMENT	64
#define MAX_WAIT	1000
#define GRAIN_WAIT	10
#define MAX_ETHERNET	1514

unsigned char mac[]={0x00,0x10,0x10,0x10,0x10,0x10,MAC_LOW_BYTE};
unsigned char packet_lora[MAX_FRAGMENT+1];
unsigned char packet_received[MAX_FRAGMENT+1];
int nb_ethernet=0;
unsigned char nb_received=0;
int index_ethernet;
unsigned char num_frag;
static int num=-1;

//control the LEDs
void output_set(int port,unsigned char value)
{
	if(value==0) PORTD &= ~(1<<port); else PORTD |= (1<<port);
}

void io_init(void)
{ 	
	DDRD |= 0x10;
	DDRD |= 0x20;
	DDRD |= 0x40;
	DDRD &= ~0x80;
}

int main(void)
{
mode           = 0x01;
Freq_Sel       = 0x00;
Power_Sel      = 0x00;
Lora_Rate_Sel  = 0x01;
BandWide_Sel   = 0x08;
Fsk_Rate_Sel   = 0x00;

io_init();
enc28j60IOSetup();
spi_init();
init_printf(9600);
printf("debug start\n");
enc28j60Init(mac);
printf("debug half\n");
sx1276_Config();
printf("debug stop\n");
unsigned char rxmode=0;
while(1)
	{
		if(nb_ethernet==0)
		{	
			nb_ethernet = enc28j60PacketReceive();
			if(nb_ethernet>0 && nb_ethernet<=MAX_ETHERNET)
			{
				index_ethernet = 0;
				num_frag = 0;
				output_set(5,1);
			}
			else nb_ethernet=0;
		}
#ifdef VERBOSE
		printf("num=%d nb_ethernet=%d\n",num,nb_ethernet);
		_delay_ms(100);
#endif
		if(num<0)
		{    
			if(nb_ethernet>0)
			{
				int i;
				num=-1; rxmode=0;
				if(index_ethernet < nb_ethernet)
				{	
					for(i=0;(i<MAX_FRAGMENT) && (index_ethernet<nb_ethernet);i++)
					{	
						unsigned char c = enc28j60ByteRead();
						index_ethernet++;
						packet_lora[i]=c;
					}
					if(index_ethernet < nb_ethernet) packet_lora[i]=0x00;    
					else packet_lora[i]=0x80;
					packet_lora[i] |= num_frag++;
#ifdef DEBUG
					printf("ether=%d/%d num_frag=%02x taille=%d\n",index_ethernet,nb_ethernet,packet_lora[i],i+1);
#endif
					sx1276_LoRaEntryTx();
					output_set(4,1);
					sx1276_LoRaTxPacket(packet_lora,i+1);  		//transmission from LoraServer
					sx1276_LoRaTxReady();
					output_set(4,0);
					if(index_ethernet >= nb_ethernet)
					{
						nb_ethernet = 0;
						enc28j60FreeMemory();
						output_set(5,0);			
						/* Attendre un paquet LoRa un temps raisonnable */
						unsigned char test=0;
						sx1276_LoRaEntryRx();
						rxmode=1;
						for(i=0;i<MAX_WAIT/GRAIN_WAIT;i++){
							test=sx1276_LoRaRxTest();
							if(test) break;
							_delay_ms(GRAIN_WAIT);
							}
						if(test){
#ifdef DEBUG
							printf("Got answer!\n");
#endif
							}
					}
					else _delay_ms(10);
				}
			}
			else{
				if(num==-2){
					unsigned char c=0;
#ifdef DEBUG
					printf("Send void packet\n");
#endif
					sx1276_LoRaEntryTx();
					sx1276_LoRaTxPacket(&c,1);
					sx1276_LoRaTxReady();
					num=-1;
				}
			}
		}
		if(index_ethernet<=0 || nb_ethernet==0){
			if(!rxmode) sx1276_LoRaEntryRx();
			rxmode=1;
			}
		if(rxmode && (nb_received=sx1276_LoRaRxPacket(packet_received,MAX_FRAGMENT+1))>0)	//receptionfrom LoraClient
		{
			unsigned char header=packet_received[nb_received-1];
#ifdef DEBUG
			printf("rec=%d h=%02x\n",nb_received,header);
#endif
			if (nb_received>1)
			{
				int j;
				if(num>=0 && (header&0x7f)!=num+1) num=-1;
				else{
					if(num<0) num=-1;
					if((header&0x7f)==0x00)
					{ 
						enc28j60InitializeSend(); 
						output_set(6,1);			
					}
					for(j=0;j<nb_received-1;j++) enc28j60WriteByteSend(packet_received[j]);
					if((header&0x80)==0x00) num++;
					else {
#ifdef DEBUG
						printf("Ether sending num=%d\n",num);
#endif
						num=-2;
						enc28j60FinalizeSend();
						output_set(6,0);
						}
					}
			}
			rxmode=0;
#ifdef DEBUG
			printf("num=%d\n",num);
#endif
		}
	}
}
