#include <stdio.h>
#include <stdint.h>
#include <avr/io.h>
#include <avr/common.h>
#include <avr/sfr_defs.h>
#include <avr/interrupt.h>

#include "SPI.h"

#define	SPI_DDR		DDRB
#define	SPI_PORT	PORTB
#define SPI_SS		2
#define SPI_MOSI	3
#define SPI_MISO	4
#define SPI_SCK		5

// Configuration of SPI IO 
void spi_setup(void) {
    DDRB |= (1<<SPI_SS);
    DDRB |= (1<<SPI_SCK);
    DDRB |= (1<<SPI_MOSI);
    DDRB &= ~(1<<SPI_MISO);
#ifdef DEBUG
    printf("SPI IO configured\n");
#endif 
}

// Initialisation of SPI bus
void spi_init(void)
{
spi_setup();
SPI_DDR |= (1<<SPI_MOSI)|(1<<SPI_SCK)|(1<<SPI_SS);
SPI_DDR &= ~(1<<SPI_MISO);
SPCR = (1<<SPE)|(1<<MSTR);
SPCR |= (1<<SPR1)|(1<<SPR0);
SPCR &= ~((1<<CPOL) | (1<<CPHA));
SPI_PORT |= (1<<SPI_SS);
#ifdef DEBUG
    printf("SPI configured\n");
#endif 
}

// Get byte from SPI bus
unsigned char spi_get(int select,unsigned char command,int cycles)
{
if(select<0) select=SPI_SS;
if(cycles<1) cycles=1;
SPI_PORT &= ~(1<<select);
SPDR = command;
while(!(SPSR & (1<<SPIF)));
int i;
for(i=0;i<cycles;i++){
	SPDR = 0;
	while(!(SPSR & (1<<SPIF)));
	}
SPI_PORT |= (1<<select);
return SPDR;
}

// Read byte on SPI bus
unsigned char spi_read(int select,unsigned char address)
{
return spi_get(select,address&0x7F,1);
}

// Send byte on SPI bus
unsigned char spi_put(int select,unsigned char command,unsigned char data)
{
if(select<0) select=SPI_SS;
SPI_PORT &= ~(1<<select);
SPDR = command;
while(!(SPSR & (1<<SPIF)));
SPDR = data;
while(!(SPSR & (1<<SPIF)));
SPI_PORT |= (1<<select);
return SPDR;
}

// Send byte on SPI bus
void spi_write(int select,unsigned char address,unsigned char data)
{
spi_put(select,address|0x80,data);
}

// Get bytes from SPI bus
void spi_get_bytes(int select,unsigned char command, unsigned char *ptr, unsigned char len)
{
if(len<1) return;
if(select<0) select=SPI_SS;
SPI_PORT &= ~(1<<select);
SPDR = command;
while(!(SPSR & (1<<SPIF)));
unsigned char i;
for (i=0; i<len; i++) {
	SPDR = 0;
	while(!(SPSR & (1<<SPIF)));
	ptr[i] = SPDR;
	}
SPI_PORT |= (1<<select);
}

// Read bytes on SPI bus
void spi_read_bytes(int select,unsigned char address, unsigned char *ptr, unsigned char len)
{
spi_get_bytes(select,address&0x7F,ptr,len);
}

// Send bytes on SPI bus
void spi_send_bytes(int select,unsigned char command, unsigned char *ptr, unsigned char len)
{
if(len<1) return;
if(select<0) select=SPI_SS;
SPI_PORT &= ~(1<<select);
SPDR = command;
while(!(SPSR & (1<<SPIF)));
unsigned char i;
for (i=0; i<len; i++) {
	SPDR = ptr[i];
	while(!(SPSR & (1<<SPIF)));
	}
SPI_PORT |= (1<<select);
}

// Write bytes on SPI bus
void spi_write_bytes(int select,unsigned char address, unsigned char *ptr, unsigned char len)
{
spi_send_bytes(select,address|0x80,ptr,len);
}
