#include <stdio.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/io.h>

#include "SPI.h"
#include "SX1276.h"

unsigned char sx1276FreqTbl[3] =
{
  0xD9,
  0x80,
  0x00,
};

//Configures the power output. Ensure bit 7 is cleared for inAir4! This is because the inAir4 uses
//the RFO output pin(low power mode), and NOT the PA_BOOST pin. 
unsigned char sx1276PowerTbl[4] =
{
  0x7F,
  0x7C,
  0x79,
  0x76,
};
 
 
unsigned char sx1276SpreadFactorTbl[7] =
{
  6,7,8,9,10,11,12
};  
 
 
unsigned char sx1276LoRaBwTbl[10] =
{
  0,1,2,3,4,5,6,7,8,9   // 7.8Khz, 10.4KHz, 15.6KHz, 20.8KHz, 31.2KHz, 41.7KHz, 62.5KHz, 125KHz, 250KHz, 500KHz
};
  
 
void sx1276_Standby(void)
{
	spi_write(-1,LR_RegOpMode, 0x01);  // standby high frequency mode
} 
 
void sx1276_Sleep(void) 
{
	spi_write(-1,LR_RegOpMode, 0x00);  // Sleep & high frequency mode
} 
 
void sx1276_EntryLoRa(void)
{
	spi_write(-1,LR_RegOpMode, 0x80);  // LoRa and Sleep & High frequency mode
}
 
void sx1276_LoRaClearIrq(void)
{
	spi_write(-1,LR_RegIrqFlags, 0xFF);
}

unsigned char sx1276_LoRaReadRSSI(void)
{
	unsigned int temp = 10;
	temp = spi_read(-1,LR_RegRssiValue);
	temp = temp + 127 - 137;
	return (unsigned char) temp;
}

unsigned char sx1276_ReadRSSI(void)
{
	unsigned int temp = 0xFF;
	 
	temp = spi_read(-1,RegRssiValue);
	temp >>= 1;
	temp = 127 - temp;
	return temp;
} 

unsigned char sx1276_ReadPktRSSI(void)
{
	unsigned int temp = 0x00;
	temp = spi_read(-1,LR_RegPktRssiValue);
	return temp;
} 
 
unsigned char sx1276_ReadPktSnr(void)
{
	unsigned int temp = 0x00;
	temp = spi_read(-1,LR_RegPktSnrValue);
	return temp;
} 

unsigned char sx1276_LoRaEntryRx(void)
{
	unsigned char ptr;
	  
	spi_write(-1,REG_LR_PADAC, 0x84);
	spi_write(-1,LR_RegHopPeriod, 0xFF);
	spi_write(-1,REG_LR_DIOMAPPING1, 0x00);
	spi_write(-1,LR_RegIrqFlagsMask, 0x3F);
	sx1276_LoRaClearIrq(); 
	spi_write(-1,LR_RegPayloadLength, 99); 
	ptr = spi_read(-1,LR_RegFifoRxBaseAddr);
	spi_write(-1,LR_RegFifoAddrPtr, ptr);
	spi_write(-1,LR_RegOpMode, 0x8d);
 	
	return 0;
} 

unsigned char sx1276_LoRaRxPacket (unsigned char *packet, int N)
{
	unsigned char addr;
	unsigned char packet_size=0;
    	unsigned char irqFlags;
	unsigned char RxDone;

	irqFlags = spi_read(-1,LR_RegIrqFlags);
	RxDone = ( irqFlags & 0x40 );				//RxDone interrupt 
#ifdef VERBOSE
	printf("f=%02x\n",irqFlags);	
#endif
	if (RxDone==0x40)  {		
		addr = spi_read(-1,LR_RegFifoRxCurrentAddr);
		spi_write(-1,LR_RegFifoAddrPtr, addr);    		// RXBaseAddr --> FiFoAddrPtr
		if (sx1276SpreadFactorTbl[Lora_Rate_Sel] == 6 ) packet_size = N;
		else packet_size = spi_read(-1,LR_RegRxNbBytes);
		spi_read_bytes(-1,0x00,packet, packet_size);
		sx1276_LoRaClearIrq();	
		return packet_size;
	  }
	else return 0; 
}

unsigned char sx1276_LoRaRxTest(void){
	unsigned char flags=spi_read(-1,LR_RegIrqFlags);
#ifdef VERBOSE
	printf("f=%02x\n",flags);	
#endif
	if((flags&0x40)!=0x00) return 1; else return 0;
} 								

void sx1276_LoRaEntryTx(void)
{
	spi_write(-1,REG_LR_PADAC, 0x84);  
	spi_write(-1,LR_RegHopPeriod, 0x00);
	spi_write(-1,REG_LR_DIOMAPPING1, 0x41);
	sx1276_LoRaClearIrq();
	spi_write(-1,LR_RegIrqFlagsMask, 0xF7);
#ifdef VERBOSE
	printf("EntryTx : ok\n");  
#endif
}

void sx1276_LoRaTxPacket(unsigned char *packet, int N)
{
	unsigned char addr; //temp;

	spi_write(-1,LR_RegPayloadLength, N);
	addr = spi_read(-1,LR_RegFifoTxBaseAddr);
	spi_write(-1,LR_RegFifoAddrPtr, addr);
#ifdef VERBOSE
	printf("Payload : %d (%d)\n",spi_read(-1,LR_RegPayloadLength),N); 
#endif
	spi_write_bytes(-1,0x00,packet,N);
#ifdef VERBOSE
	printf("%d bytes sent\n",N); 
#endif
	spi_write(-1,LR_RegOpMode,0x83);         					    	
#ifdef VERBOSE
	printf("Sending done\n"); 
#endif
}

unsigned char sx1276_LoRaTxReady(void)			
{
	unsigned char TxDone;
	  
	while(1) {
		TxDone = (PIND & 0x80)>>7;
		if (TxDone)  { 
#ifdef VERBOSE
			printf("TxDone : %x\n",TxDone);   
#endif
	      		spi_read(-1,LR_RegIrqFlags);
	      		sx1276_LoRaClearIrq();
	      		sx1276_Standby();
	      		break;
    	    	}
  	  }
return 0;
}
 
void sx1276_Config(void)
{
	sx1276_Sleep();
#ifdef DEBUG
	printf("ModeSleep is 0x%02x\n",spi_read(-1,LR_RegOpMode));
#endif
	_delay_ms(15);
	 
	sx1276_EntryLoRa();
	//spi_write(-1,0x5904); // change digital regulator from 1.6V to 1.47V: see errata note
	spi_write_bytes(-1,LR_RegFrMsb, (unsigned char *)sx1276FreqTbl,3);		// set the frequency parameter
	spi_write(-1,LR_RegPaConfig, sx1276PowerTbl[Power_Sel]); 			// set the output power parameter
	spi_write(-1,LR_RegOcp, 0x0B); 						// overload current protection
	spi_write(-1,LR_RegLna, 0x23);						// default LNA current + maximum gain+boost on 150%current
	 
	if(sx1276SpreadFactorTbl[Lora_Rate_Sel]==6) { 
		unsigned char tmp;
		spi_write(-1,LR_RegModemConfig1, ((sx1276LoRaBwTbl[BandWide_Sel] << 4)+(CR<<1)+0x01));  
		// Implicit Enable CRC Enable (0x02) & Error Coding rate 4/5 (0x01), 4/6 (0x02), 4/7 (0x03), 4/8 (0x04)
		spi_write(-1,LR_RegModemConfig2, ((sx1276SpreadFactorTbl[Lora_Rate_Sel]<<4)+(CRC<<2)+0x03));
		// Settings of SF=6
	    	tmp = spi_read(-1,0x31);
	    	tmp &= 0xF8;
	    	tmp |= 0x05;
	    	spi_write(-1,0x31,tmp);
	    	spi_write(-1,0x37, 0x0C);
	  	}
	  else {
#ifdef DEBUG
		printf("%s\n","config - elseif loop");
#endif
	    	spi_write(-1,LR_RegModemConfig1,((sx1276LoRaBwTbl[BandWide_Sel] <<4 )+(CR<<1)+0x00));
	    	spi_write(-1,LR_RegModemConfig2, ((sx1276SpreadFactorTbl[Lora_Rate_Sel]<<4)+ (CRC<<2)+0x03));
		}

	spi_write(-1,LR_RegSymbTimeoutLsb, 0xFF);
	spi_write(-1,LR_RegPreambleMsb, 0x00);
	spi_write(-1,LR_RegPreambleLsb, 12);
	spi_write(-1,REG_LR_DIOMAPPING2, 0x01);
	sx1276_Standby();
#ifdef DEBUG
	printf("Config - finished method, forced Opmode to Standby, Opmode is now :0x%2x\n",spi_read(-1,LR_RegOpMode)); 
#endif
} 
